from .helpers import generate_attribute_string
from .constants import LikeStatus, RepeatType


class Player:
    """Represent Player state."""

    def __init__(self, raw, request) -> None:
        self._raw = raw
        self._request = request

    def __str__(self) -> str:
        attributes = [
            "has_song",
            "is_paused",
            "volume_percent",
            "seekbar_current_position",
            "seekbar_current_position_human",
            "state_percent",
            "like_status",
            "repeat_type",
        ]
        return generate_attribute_string(self, attributes)

    def __eq__(self, other: object) -> bool:
        return self._raw == other._raw

    @property
    def has_song(self) -> bool:
        return self._raw["hasSong"]

    @property
    def is_paused(self) -> bool:
        return self._raw["isPaused"]

    @property
    def volume_percent(self) -> int:
        return self._raw["volumePercent"]

    @property
    def seekbar_current_position(self) -> int:
        return self._raw["seekbarCurrentPosition"]

    @property
    def seekbar_current_position_human(self) -> str:
        return self._raw["seekbarCurrentPositionHuman"]

    @property
    def state_percent(self) -> int:
        return self._raw["statePercent"]

    @property
    def like_status(self) -> LikeStatus:
        return self._raw["likeStatus"]

    @property
    def repeat_type(self) -> RepeatType:
        return self._raw["repeatType"]

    async def update(self) -> None:
        response = await self._request("get", "/player")
        if response:
            self._raw = response



from enum import Enum


class LikeStatus(str, Enum):
    DISLIKE = "DISLIKE"
    INDIFFERENT = "INDIFFERENT"
    LIKE = "LIKE"


class RepeatType(str, Enum):
    NONE = "NONE"
    ONE = "ONE"
    ALL = "ALL"



from .helpers import generate_attribute_string

class Track:
    """Represent Track state."""

    def __init__(self, raw, request) -> None:
        self._raw = raw
        self._request = request

    def __str__(self) -> str:
        attributes = [
            "author",
            "title",
            "album",
            "cover",
            "duration",
            "duration_human",
            "url",
            "id",
            "is_video",
            "is_advertisement",
            "in_library",
        ]
        return generate_attribute_string(self, attributes)

    def __eq__(self, other: object) -> bool:
        return self._raw == other._raw

    @property
    def author(self) -> str:
        return self._raw["author"]

    @property
    def title(self) -> str:
        return self._raw["title"]

    @property
    def album(self) -> str:
        return self._raw["album"]

    @property
    def cover(self) -> str:
        return self._raw["cover"]

    @property
    def duration(self) -> int:
        return self._raw["duration"]

    @property
    def duration_human(self) -> str:
        return self._raw["durationHuman"]

    @property
    def url(self) -> str:
        return self._raw["url"]

    @property
    def id(self) -> str:
        return self._raw["id"]

    @property
    def is_video(self) -> bool:
        return self._raw["isVideo"]

    @property
    def is_advertisement(self) -> bool:
        return self._raw["isAdvertisement"]

    @property
    def in_library(self) -> bool:
        return self._raw["inLibrary"]

    async def update(self) -> None:
        response = await self._request("get", "/track")
        if response:
            self._raw = response



from .constants import RepeatType


class SendCommand:
    def __init__(
        self,
        request,
    ) -> None:
        self._request = request

    async def _send_command(self, command, value=None):
        data = {"command": command}
        if value:
            data["value"] = value
        await self._request("POST", "", data=data)

    async def show_lyrics_hidden(self):
        """Open hidden lyrics window to provide data"""
        await self._send_command("show-lyrics-hidden")

    async def track_play(self):
        """Play music"""
        await self._send_command("track-play")

    async def track_pause(self):
        """Pause music"""
        await self._send_command("track-pause")

    async def track_next(self):
        """Next track"""
        await self._send_command("track-next")

    async def track_previous(self):
        """Previous track"""
        await self._send_command("track-previous")

    async def track_thumbs_up(self):
        """Like current track"""
        await self._send_command("track-thumbs-up")

    async def track_thumbs_down(self):
        """Dislike current track"""
        await self._send_command("track-thumbs-down")

    async def player_volume_up(self):
        """Increase the player volume"""
        await self._send_command("player-volume-up")

    async def player_volume_down(self):
        """Decrease the player volume"""
        await self._send_command("player-volume-down")

    async def player_forward(self):
        """Forward 10 seconds"""
        await self._send_command("player-forward")

    async def player_rewind(self):
        """Rewind 10 seconds"""
        await self._send_command("player-rewind")

    async def player_repeat(self, repeat_mode: RepeatType):
        """Toggle NONE or ONE or ALL"""
        await self._send_command("player-repeat", repeat_mode)

    async def player_shuffle(self):
        """Shuffle queue"""
        await self._send_command("player-shuffle")

    async def player_add_library(self):
        """Add track to library"""
        await self._send_command("player-add-library")

    async def player_add_library(self, playlist_index):
        """Add track to playlist (Needs value: Playlist index)"""
        await self._send_command("player-add-playlist", playlist_index)

    async def player_set_seekbar(self, value):
        """Set value for seekbar (Needs value: 0 ~ Track duration)"""
        await self._send_command("player-set-seekbar", value)

    async def player_set_volume(self, value):
        """Set value for volume (Needs value: 0 ~ 100)"""
        await self._send_command("player-set-volume", value)

    async def player_set_queue(self, value):
        """Set index of queue to play track (Needs value: 0 ~ Queue length)"""
        await self._send_command("player-set-queue", value)
